 /***********************************************************************
 *                                                                      *
 * boot.c                                                               *
 * This is vector and interrupt program with C.                         *
 * History:  2009/01/14  Start.                                         *
 *                                                                      *
 ***********************************************************************/

/* #include */
#include <string.h>
#include "init.h"

/* Prototype */
void boot(void);
void clearBss(void);
void copyLmaToVma(void);
void end(void);
void dummy(void)__attribute__ ((interrupt_handler));
void nmi(void)__attribute__ ((interrupt_handler));

/* extern */
extern int main(void);
extern void t16ch0Int(void)__attribute__ ((interrupt_handler));
extern void t16ch1Int(void)__attribute__ ((interrupt_handler));
extern void p0xInt(void)__attribute__ ((interrupt_handler));
extern void display2lcd(void)__attribute__ ((interrupt_handler));  //**TK**
extern unsigned long __START_bss;
extern unsigned long __END_bss;
extern unsigned long __START_data;
extern unsigned long __START_data_lma;
extern unsigned long __END_data;
extern void emu_copro_process(void);
/* special definitions for vector table  */
typedef void func(void);
#ifdef __POINTER16
#define VECTOR(vec)  vec,0
#else
#define VECTOR(vec)  vec
#endif

/* vector table */
func *const vector[] = {		// No	Base+	Name
	VECTOR(boot),				// 0	00		Reset
	VECTOR(dummy),				// 1	04		Illegal Adress Int.
	VECTOR(nmi),				// 2	08		NMI
	VECTOR(emu_copro_process),	// 3	0c		reserved
	VECTOR(p0xInt),				// 4	10		P0 Int.
	VECTOR(dummy),				// 5	14		P1 Int.
	VECTOR(dummy),				// 6	18		SWT Int.
	VECTOR(dummy),				// 7	1c		CT Int.
	VECTOR(dummy),				// 8	20		T8OSC1 Int.
	VECTOR(dummy),				// 9	24		SVD Int.
	VECTOR(display2lcd),				// 10	28		LCD Int.  //**TK**
	VECTOR(dummy),				// 11	2c		T16E Ch.0 Int.
	VECTOR(dummy),				// 12	30		T8F Int.
	VECTOR(t16ch0Int),			// 13	34		T16 Ch.0 Int.
	VECTOR(t16ch1Int),			// 14	38		T16 Ch.1 Int.
	VECTOR(dummy),				// 15	3c		T16 Ch.2 Int.
	VECTOR(dummy),				// 16	40		UART Ch.0 Int.
	VECTOR(dummy),				// 17	44		UART Ch.1 Int.
	VECTOR(dummy),				// 18	48		SPI Int.
	VECTOR(dummy),				// 19	4c		I2C Int.
	VECTOR(dummy),				// 20	50		REMC Int.
	VECTOR(dummy),				// 21	54		T16E Ch.1 Int.
	VECTOR(0),					// 22	58		reserved
	VECTOR(0),					// 23	5c		reserved
	VECTOR(0),					// 24	60		reserved
	VECTOR(0),					// 25	64		reserved
	VECTOR(0),					// 26	68		reserved
	VECTOR(0),					// 27	6c		reserved
	VECTOR(0),					// 28	70		reserved
	VECTOR(0),					// 29	74		reserved
	VECTOR(0),					// 30	78		reserved
	VECTOR(0)					// 32	7C		MAGIC No.
//	VECTOR((void *)0xec170000)	// 32	7C		MAGIC No.
};

/**********************************************************************
 * Boot function.
 * First, it resets the Stack Pointer, clear the bss area, and copy LMA to VMA.
 * Next, it calls "setMiscAccessCycle" function.
 * Next, it permit interrput enable of Processor States Register.
 * Then, it calls "main" function and calls "end" function.
 *
 * return: void
 * history: 2008/03/28	start
 **********************************************************************/
void boot(void) {
	asm("xld.a  %sp, 0xfc0");  	// Set SP in RAM

	clearBss();						// Clear the bss area
	copyLmaToVma();					// Copy LMA data to VMA data

	controlClg(CLG_PCLKEN_ENA);     //run clg
    setMiscAccessCycle(MISC_FACCESS_CYCLE_1, MISC_SACCESS_CYCLE_2);	// Set read cycles

	asm("ei");						// Interrupt enable

	main(); 						// Call main
	end(); 							// Call end
}

/**********************************************************************
 * ClearBss function.
 * This function clears the bss area.
 *
 * return: void
 * history: 2008/03/26	start
 **********************************************************************/
void clearBss(void) {
	unsigned long ulCnt;
	unsigned char *pcData;

	pcData = (unsigned char *)&__START_bss;
	for( ulCnt = 0; ulCnt < ((unsigned char *)&__END_bss - (unsigned char *)&__START_bss); ++ulCnt ) {
		*pcData = 0;
		++pcData;
	}
}


/**********************************************************************
 * copyLmaToVma function.
 * This function copys LMA data to VMA data.
 *
 * return: void
 * history: 2008/03/26	start
 **********************************************************************/
void copyLmaToVma(void) {
	memcpy((unsigned char *)&__START_data, (unsigned char *)&__START_data_lma, ((unsigned char *)&__END_data - (unsigned char *)&__START_data));
}


/**********************************************************************
 * End function.
 * This function has only infinite loop.
 *
 * return: void
 * history: 2008/03/26	start
 **********************************************************************/
void end(void) {
	// infinite loop
	while(1) {
	}
}


/**********************************************************************
 * Non Maskable Interrupt function.
 *
 * return: void
 * history: 2008/03/26	start
 **********************************************************************/
void nmi(void) {
	// Processing is undecided.
}


/**********************************************************************
 * Dummy Interrupt function.
 * This function has only infinite loop.
 *
 * return: void
 * history: 2008/03/26	start
 **********************************************************************/
void dummy(void) {
	// infinite loop
	while(1) {
	}
}
